"""
	Maximum flow -- Ford-Fulkerson algorithm
	O((n+m) OPT) time, O(n+m) space
"""
import math
from FLOWNET import flow_network

def augment(G, seen, u, t, B): 
	"""Augment along any u-t path in the residual graph of G, avoiding already visited (seen) 
	nodes and assuming a preexisting capacity bottleneck of B. 
	Return the bottleneck capacity of the path (or 0 if no path)"""
	if u == t: return B
	seen[u] = True
	for e in G[u]: 
		if not seen[e.head] and e.cap - e.f > 0: 
			df = augment(G, seen, e.head, t, min(B, e.cap - e.f))
			if df > 0: 
				e.f = e.f + df	# aggiorno il flusso su e
				G[e.head][e.rev].f = G[e.head][e.rev].f - df # aggiorno anche l'arco rovescio di e!
				return df
	return 0

def max_flow(G, s, t): 
	"Return the maximum value of an s-t flow in the graph G"
	mf = 0 #variabile che mantiene il valore del flusso trovato fino ad ora
	while True: 
		seen = len(G) * [False]
		df = augment(G, seen, s, t, math.inf) 
		mf = mf + df
		if df == 0: break
	return mf
	
def main():
	n, m = [int(s) for s in input().split()]
	G = flow_network(n+1) # da 0 fino a n incluso
	while m > 0: 
		u, v, cap = [int(s) for s in input().split()]
		G.add_arc(u, v, cap)
		m = m - 1
	print(max_flow(G, 1, n))

if __name__ == '__main__': main()
